using Godot;
using Godot.Collections;


public class ServerCube : Spatial {
  public Array<SpatialMaterial> Materials;
  RID Scenario;
  RID Space;

  public override void _Ready() {
    Materials = new Array<SpatialMaterial>();
    Scenario = GetWorld().Scenario;
    Space = GetWorld().Space;

    this.RenderCube(1f, 1f, new Vector3(2f, 0f, 0f));
  }

  public void RenderCube(float width, float height, Vector3 position) {
    SpatialMaterial material = new SpatialMaterial();
    material.AlbedoColor = new Color("#ec6161");
    RID Material = material.GetRid();
    Materials.Add(material);

    RID instance = VisualServer.InstanceCreate();
    VisualServer.InstanceSetScenario(instance, Scenario);

    RID mesh = VisualServer.MeshCreate();
    VisualServer.MeshAddSurfaceFromArrays(mesh, VisualServer.PrimitiveType.Triangles, this.GetCubeMeshArrays(width, height));
    VisualServer.MeshSurfaceSetMaterial(mesh, 0, Material);
    VisualServer.InstanceSetBase(instance, mesh);

    Transform transform = new Transform(Basis.Identity, position);
    VisualServer.InstanceSetTransform(instance, transform);

    // physics
    RID body = PhysicsServer.BodyCreate();
    PhysicsServer.BodySetMode(body, PhysicsServer.BodyMode.Static);
    
    GD.Print("-- BODY --");
    Debug.Log(body);

    RID shape = PhysicsServer.ShapeCreate(PhysicsServer.ShapeType.Box);
    PhysicsServer.ShapeSetData(shape, new Vector3(width / 2f, height / 2f, width / 2f));
    PhysicsServer.BodyAddShape(body, shape, new Transform(Basis.Identity, new Vector3(0f, height / 2f, 0f)));

    GD.Print("-- SHAPE --");
    Debug.Log(shape);

    PhysicsServer.BodySetSpace(body, Space);

    PhysicsServer.BodySetState(body, PhysicsServer.BodyState.Transform, transform);
  }

  public Array GetCubeMeshArrays(float width, float height) {
    float offset = width / 2;

    Vector3[] vertices = new Vector3[] {
      // top 4
      new Vector3(0 - offset, height, 0 - offset),
      new Vector3(0 + offset, height, 0 - offset),
      new Vector3(0 + offset, height, 0 + offset),
      new Vector3(0 - offset, height, 0 + offset),
      // bottom 4
      new Vector3(0 - offset, 0, 0 - offset),
      new Vector3(0 + offset, 0, 0 - offset),
      new Vector3(0 + offset, 0, 0 + offset),
      new Vector3(0 - offset, 0, 0 + offset),
    };

    int[] indices = new int[] {
      // roof
      0, 1, 2, 
      0, 2, 3,
      // -z side
      4, 5, 1,
      4, 1, 0,
      // z side
      6, 7, 3,
      6, 3, 2,
      // x side
      5, 6, 2,
      5, 2, 1,
      // -x side
      7, 4, 0,
      7, 0, 3
    };

    Array arrays = new Array();
    arrays.Resize((int)Mesh.ArrayType.Max);

    arrays[(int)Mesh.ArrayType.Vertex] = vertices;
    arrays[(int)Mesh.ArrayType.Index] = indices;

    return arrays;
  }
}